@@ -59,6 +59,7 @@
 #include <iterator>
 #include <functional>
 #include <numeric>
+#include <o3tl/compat_functional.hxx>
 
 using namespace ::com::sun::star;
 using namespace ::com::sun::star::chart2;
@@ -122,10 +123,10 @@ OUString lcl_ConvertRole( const OUString & rRoleString, bool bFromInternalToUI )
     {
         tTranslationMap::const_iterator aIt(
             ::std::find_if( aTranslationMap.begin(), aTranslationMap.end(),
-                            ::std::compose1( ::std::bind2nd(
+                            ::o3tl::compose1( ::std::bind2nd(
                                                  ::std::equal_to< tTranslationMap::mapped_type >(),
                                                  rRoleString ),
-                                             ::std::select2nd< tTranslationMap::value_type >())));
+                                             ::o3tl::select2nd< tTranslationMap::value_type >())));
 
         if( aIt != aTranslationMap.end())
             aResult = (*aIt).first;
@@ -161,7 +162,7 @@ void lcl_createRoleIndexMap( lcl_tRoleIndexMap & rOutMap )
 struct lcl_DataSeriesContainerAppend : public
     ::std::iterator< ::std::output_iterator_tag, Reference< XDataSeriesContainer > >
 {
-    typedef ::std::vector< chart::DialogModel::tSeriesWithChartTypeByName > tContainerType;
+    typedef ::std::vector< ::chart::DialogModel::tSeriesWithChartTypeByName > tContainerType;
 
     explicit lcl_DataSeriesContainerAppend( tContainerType & rCnt )
             : m_rDestCnt( rCnt )
@@ -181,7 +182,7 @@ struct lcl_DataSeriesContainerAppend : public
                 for( sal_Int32 nI = 0; nI < aSeq.getLength(); ++ nI )
                 {
                     m_rDestCnt.push_back(
-                        chart::DialogModel::tSeriesWithChartTypeByName(
+                        ::chart::DialogModel::tSeriesWithChartTypeByName(
                             ::chart::DataSeriesHelper::getDataSeriesLabel( aSeq[nI], aRole ),
                             ::std::make_pair( aSeq[nI], xCT )));
                 }
@@ -382,7 +383,6 @@ DialogModel::DialogModel(
         m_xContext( xContext ),
         m_aTimerTriggeredControllerLock( uno::Reference< frame::XModel >( m_xChartDocument, uno::UNO_QUERY ) )
 {
-    createBackup();
 }
 
 DialogModel::~DialogModel()
@@ -688,7 +688,7 @@ bool DialogModel::setData(
     if( ! xDataProvider.is() ||
         ! m_xTemplate.is() )
     {
-        OSL_ENSURE( false, "Model objects missing" );
+        OSL_FAIL( "Model objects missing" );
         return false;
     }
 
@@ -725,20 +725,16 @@ bool DialogModel::setData(
     return true;
 }
 
-// static
 OUString DialogModel::ConvertRoleFromInternalToUI( const OUString & rRoleString )
 {
     return lcl_ConvertRole( rRoleString, true );
 }
 
-// static
 OUString DialogModel::GetRoleDataLabel()
 {
     return OUString( String( ::chart::SchResId( STR_OBJECT_DATALABELS )));
 }
 
-// static
-
 sal_Int32 DialogModel::GetRoleIndexForSorting( const ::rtl::OUString & rInternalRoleString )
 {
     static lcl_tRoleIndexMap aRoleIndexMap;
@@ -755,20 +751,6 @@ sal_Int32 DialogModel::GetRoleIndexForSorting( const ::rtl::OUString & rInternal
 
 // private methods
 
-void DialogModel::createBackup()
-{
-    OSL_ENSURE( ! m_xBackupChartDocument.is(), "Cloning already cloned model" );
-    try
-    {
-        Reference< util::XCloneable > xCloneable( m_xChartDocument, uno::UNO_QUERY_THROW );
-        m_xBackupChartDocument.set( xCloneable->createClone(), uno::UNO_QUERY_THROW );
-    }
-    catch( uno::Exception & ex )
-    {
-        ASSERT_EXCEPTION( ex );
-    }
-}
-
 void DialogModel::applyInterpretedData(
     const InterpretedData & rNewData,
     const ::std::vector< Reference< XDataSeries > > & rSeriesToReUse,
