@@ -1402,7 +1402,12 @@ void ScTable::UpdateGrow( const ScRange& rArea, SCCOL nGrowX, SCROW nGrowY )
 
 void ScTable::UpdateInsertTab(SCTAB nTable)
 {
-    if (nTab >= nTable) nTab++;
+    if (nTab >= nTable)
+    {
+        nTab++;
+        if (pDBDataNoName)
+            pDBDataNoName->UpdateMoveTab(nTab - 1 ,nTab);
+    }
     for (SCCOL i=0; i <= MAXCOL; i++) aCol[i].UpdateInsertTab(nTable);
 
     if (IsStreamValid())
@@ -1411,7 +1416,12 @@ void ScTable::UpdateInsertTab(SCTAB nTable)
 
 void ScTable::UpdateDeleteTab( SCTAB nTable, sal_Bool bIsMove, ScTable* pRefUndo )
 {
-    if (nTab > nTable) nTab--;
+    if (nTab > nTable)
+    {
+        nTab--;
+        if (pDBDataNoName)
+            pDBDataNoName->UpdateMoveTab(nTab + 1,nTab);
+    }
 
     SCCOL i;
     if (pRefUndo)
@@ -1696,6 +1706,8 @@ void ScTable::SetRepeatColRange( const ScRange* pNew )
 
     if (IsStreamValid())
         SetStreamValid(false);
+
+    InvalidatePageBreaks();
 }
 
 void ScTable::SetRepeatRowRange( const ScRange* pNew )
@@ -1704,6 +1716,8 @@ void ScTable::SetRepeatRowRange( const ScRange* pNew )
 
     if (IsStreamValid())
         SetStreamValid(false);
+
+    InvalidatePageBreaks();
 }
 
 void ScTable::ClearPrintRanges()
@@ -1713,6 +1727,8 @@ void ScTable::ClearPrintRanges()
 
     if (IsStreamValid())
         SetStreamValid(false);
+
+    InvalidatePageBreaks();
 }
 
 void ScTable::AddPrintRange( const ScRange& rNew )
@@ -1723,6 +1739,8 @@ void ScTable::AddPrintRange( const ScRange& rNew )
 
     if (IsStreamValid())
         SetStreamValid(false);
+
+    InvalidatePageBreaks();
 }
 
 
