@@ -57,6 +57,10 @@
 #include "drwlayer.hxx"
 #include "userdat.hxx"
 #include "scmod.hxx"
+#include "client.hxx"
+#include "charthelper.hxx"
+#include "docuno.hxx"
+#include "docsh.hxx"
 
 // -----------------------------------------------------------------------
 
@@ -81,7 +85,7 @@ using namespace com::sun::star;
 FuSelection::FuSelection(ScTabViewShell* pViewSh, Window* pWin, ScDrawView* pViewP,
                SdrModel* pDoc, SfxRequest& rReq ) :
     FuDraw(pViewSh, pWin, pViewP, pDoc, rReq),
-    bVCAction(FALSE)
+    bVCAction(false)
 {
 }
 
@@ -95,7 +99,7 @@ FuSelection::~FuSelection()
 {
 }
 
-BYTE FuSelection::Command(const CommandEvent& rCEvt)
+sal_uInt8 FuSelection::Command(const CommandEvent& rCEvt)
 {
     //	special code for non-VCL OS2/UNX removed
 
@@ -108,22 +112,22 @@ BYTE FuSelection::Command(const CommandEvent& rCEvt)
 |*
 \************************************************************************/
 
-BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
+sal_Bool FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
 {
-    // #95491# remember button state for creation of own MouseEvents
+    // remember button state for creation of own MouseEvents
     SetMouseButtonCode(rMEvt.GetButtons());
     const bool bSelectionOnly = rMEvt.IsRight();
     if ( pView->IsAction() )
     {
         if ( bSelectionOnly )
             pView->BckAction();
-        return TRUE;
+        return sal_True;
     }
 
-    bVCAction = FALSE;
-    bIsInDragMode = FALSE;		//	irgendwo muss es ja zurueckgesetzt werden (#50033#)
+    bVCAction = false;
+    bIsInDragMode = false;		//	irgendwo muss es ja zurueckgesetzt werden (#50033#)
 
-    BOOL bReturn = FuDraw::MouseButtonDown(rMEvt);
+    sal_Bool bReturn = FuDraw::MouseButtonDown(rMEvt);
 
     aMDPos = pWindow->PixelToLogic( rMEvt.GetPosPixel() );
 
@@ -163,16 +167,16 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
             {
                 aDragTimer.Start();
                 pView->BegDragObj(aMDPos, (OutputDevice*) NULL, pHdl);
-                bReturn = TRUE;
+                bReturn = sal_True;
             }
         }
         else
         {
-            BOOL bAlt = rMEvt.IsMod2();
+            sal_Bool bAlt = rMEvt.IsMod2();
             if ( !bAlt && pView->PickObj(aMDPos, pView->getHitTolLog(), pObj, pPV, SDRSEARCH_PICKMACRO) )
             {
                 pView->BegMacroObj(aMDPos, pObj, pPV, pWindow);
-                bReturn = TRUE;
+                bReturn = sal_True;
             }
             else
             {
@@ -202,7 +206,7 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
                            pObj = pHit;
                    }
 
-                   ScMacroInfo* pInfo = ScDrawLayer::GetMacroInfo( pObj, TRUE );
+                   ScMacroInfo* pInfo = ScDrawLayer::GetMacroInfo( pObj, true );
                    // For interoperability favour links over macros if both are defined
                    if ( pInfo->GetHlink().getLength() > 0 )
                    {
@@ -220,7 +224,7 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
                            {
                                try
                                {
-                                   aCaller = xProps->getPropertyValue( rtl::OUString::createFromAscii("Name") );
+                                   aCaller = xProps->getPropertyValue( rtl::OUString(RTL_CONSTASCII_USTRINGPARAM("Name")) );
                                }
                                catch( uno::Exception& ) {}
                            }
@@ -232,7 +236,7 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
                            pObjSh->CallXScript( pInfo->GetMacro(),
                                *pInArgs, aRet, aOutArgsIndex, aOutArgs, true, &aCaller );
                            pViewShell->FakeButtonUp( pViewShell->GetViewData()->GetActivePart() );
-                           return TRUE;        // kein CaptureMouse etc.
+                           return sal_True;        // kein CaptureMouse etc.
                        }
                    }
                 }
@@ -266,13 +270,13 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
                 {
                     ScGlobal::OpenURL( sURL, sTarget );
                     pViewShell->FakeButtonUp( pViewShell->GetViewData()->GetActivePart() );
-                    return TRUE;        // kein CaptureMouse etc.
+                    return sal_True;        // kein CaptureMouse etc.
                 }
 
                 //	Is another object being edited in this view?
                 //	(Editing is ended in MarkListHasChanged - test before UnmarkAll)
                 SfxInPlaceClient* pClient = pViewShell->GetIPClient();
-                BOOL bWasOleActive = ( pClient && pClient->IsObjectInPlaceActive() );
+                sal_Bool bWasOleActive = ( pClient && pClient->IsObjectInPlaceActive() );
 
                 //	Markieren
 
@@ -287,14 +291,14 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
                     pView->UnlockInternalLayer();
 
                 // try to select the clicked object
-                if ( pView->MarkObj( aMDPos, -2, FALSE, rMEvt.IsMod1() ) )
+                if ( pView->MarkObj( aMDPos, -2, false, rMEvt.IsMod1() ) )
                 {
                     //*********************************************************
                     //Objekt verschieben
                     //********************************************************
                     if (pView->IsMarkedHit(aMDPos))
                     {
-                        //	#95834# Don't start drag timer if inplace editing of an OLE object
+                        //	Don't start drag timer if inplace editing of an OLE object
                         //	was just ended with this mouse click - the view will be moved
                         //	(different tool bars) and the object that was clicked on would
                         //	be moved unintentionally.
@@ -303,11 +307,11 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
 
                         pHdl=pView->PickHandle(aMDPos);
                         pView->BegDragObj(aMDPos, (OutputDevice*) NULL, pHdl);
-                        bReturn = TRUE;
+                        bReturn = sal_True;
                     }
                     else									// Objekt am Rand getroffen
                         if (pViewShell->IsDrawSelMode())
-                            bReturn = TRUE;
+                            bReturn = sal_True;
                 }
                 else
                 {
@@ -319,7 +323,7 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
                         //Objekt selektieren
                         //********************************************************
                         pView->BegMarkObj(aMDPos);
-                        bReturn = TRUE;
+                        bReturn = sal_True;
                     }
                 }
             }
@@ -343,9 +347,9 @@ BOOL __EXPORT FuSelection::MouseButtonDown(const MouseEvent& rMEvt)
 |*
 \************************************************************************/
 
-BOOL __EXPORT FuSelection::MouseMove(const MouseEvent& rMEvt)
+sal_Bool FuSelection::MouseMove(const MouseEvent& rMEvt)
 {
-    BOOL bReturn = FuDraw::MouseMove(rMEvt);
+    sal_Bool bReturn = FuDraw::MouseMove(rMEvt);
 
     if (aDragTimer.IsActive() )
     {
@@ -363,20 +367,14 @@ BOOL __EXPORT FuSelection::MouseMove(const MouseEvent& rMEvt)
 
         ForceScroll(aPix);
         pView->MovAction(aPnt);
-        bReturn = TRUE;
+        bReturn = sal_True;
     }
 
     // Ein VCControl ist aktiv
     // Event an den Manager weiterleiten
     if( bVCAction )
     {
-        //	GetSbxForm gibts nicht mehr - Basic-Controls sind tot
-        //SdrPageView* pPgView = pView->GetPageViewByIndex(0);
-        //ScDrawPage*  pPage	 = (ScDrawPage*)pPgView->GetPage();
-        //VCSbxForm* pForm = (VCSbxForm*)(SbxObject*)(pPage->GetSbxForm());
-        //((VCManager*)(pForm->GetVCContainer()))->
-        //    MouseMove( pWindow, rMEvt );
-        bReturn = TRUE;
+        bReturn = true;
     }
 
     ForcePointer(&rMEvt);
@@ -390,14 +388,13 @@ BOOL __EXPORT FuSelection::MouseMove(const MouseEvent& rMEvt)
 |*
 \************************************************************************/
 
-BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
+sal_Bool FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
 {
-    // #95491# remember button state for creation of own MouseEvents
+    // remember button state for creation of own MouseEvents
     SetMouseButtonCode(rMEvt.GetButtons());
 
-    BOOL bReturn = FuDraw::MouseButtonUp(rMEvt);
-//	BOOL bOle	 = pViewShell->GetViewData()->IsOle();
-    BOOL bOle = pViewShell->GetViewFrame()->GetFrame().IsInPlace();
+    sal_Bool bReturn = FuDraw::MouseButtonUp(rMEvt);
+    sal_Bool bOle = pViewShell->GetViewFrame()->GetFrame().IsInPlace();
 
     if (aDragTimer.IsActive() )
     {
@@ -406,6 +403,14 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
 
     Point aPnt( pWindow->PixelToLogic( rMEvt.GetPosPixel() ) );
 
+    bool bCopy = false;
+    ScViewData* pViewData = ( pViewShell ? pViewShell->GetViewData() : NULL );
+    ScDocument* pDocument = ( pViewData ? pViewData->GetDocument() : NULL );
+    SdrPageView* pPageView = ( pView ? pView->GetSdrPageView() : NULL );
+    SdrPage* pPage = ( pPageView ? pPageView->GetPage() : NULL );
+    ::std::vector< ::rtl::OUString > aExcludedChartNames;
+    ScRangeListVector aProtectedChartRangesVector;
+
     if ( rMEvt.IsLeft() )
     {
         if ( pView->IsDragObj() )
@@ -413,6 +418,29 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
             /******************************************************************
             * Objekt wurde verschoben
             ******************************************************************/
+            if ( rMEvt.IsMod1() )
+            {
+                if ( pPage )
+                {
+                    ScChartHelper::GetChartNames( aExcludedChartNames, pPage );
+                }
+                if ( pView && pDocument )
+                {
+                    const SdrMarkList& rSdrMarkList = pView->GetMarkedObjectList();
+                    sal_uLong nMarkCount = rSdrMarkList.GetMarkCount();
+                    for ( sal_uLong i = 0; i < nMarkCount; ++i )
+                    {
+                        SdrMark* pMark = rSdrMarkList.GetMark( i );
+                        SdrObject* pObj = ( pMark ? pMark->GetMarkedSdrObj() : NULL );
+                        if ( pObj )
+                        {
+                            ScChartHelper::AddRangesIfProtectedChart( aProtectedChartRangesVector, pDocument, pObj );
+                        }
+                    }
+                }
+                bCopy = true;
+            }
+
             pView->EndDragObj( rMEvt.IsMod1() );
             pView->ForceMarkedToAnotherPage();
 
@@ -425,7 +453,7 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
                   FuText* pText = static_cast<FuText*>(pPoor);
                 pText->StopDragMode(pObj );
             }
-            bReturn = TRUE;
+            bReturn = sal_True;
         }
         else if (pView->IsAction() )
         {
@@ -434,16 +462,16 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
             pView->EndAction();
             if ( pView->AreObjectsMarked() )
             {
-                bReturn = TRUE;
+                bReturn = sal_True;
 
                 /*  if multi-selection contains a note caption object, remove
                     all other objects from selection. */
                 const SdrMarkList& rMarkList = pView->GetMarkedObjectList();
-                ULONG nCount = rMarkList.GetMarkCount();
+                sal_uLong nCount = rMarkList.GetMarkCount();
                 if( nCount > 1 )
                 {
                     bool bFound = false;
-                    for( ULONG nIdx = 0; !bFound && (nIdx < nCount); ++nIdx )
+                    for( sal_uLong nIdx = 0; !bFound && (nIdx < nCount); ++nIdx )
                     {
                         SdrObject* pObj = rMarkList.GetMark( nIdx )->GetMarkedSdrObj();
                         bFound = ScDrawLayer::IsNoteCaption( pObj );
@@ -458,14 +486,6 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
         }
     }
 
-/*
-    if ( pView->IsObjEdit() )
-    {
-        BOOL bShowCursor = TRUE;
-//! 	pOutlinerView = pView->GetOutlinerView(pWindow, bShowCursor);
-        bReturn = TRUE;
-    }
-*/
     /**************************************************************************
     * Ggf. OLE-Objekt beruecksichtigen
     **************************************************************************/
@@ -480,7 +500,7 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
             pIPClient->DeactivateObject();
     }
 
-    USHORT nClicks = rMEvt.GetClicks();
+    sal_uInt16 nClicks = rMEvt.GetClicks();
     if ( nClicks == 2 && rMEvt.IsLeft() )
     {
         if ( pView->AreObjectsMarked() )
@@ -491,14 +511,14 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
                 SdrMark* pMark = rMarkList.GetMark(0);
                 SdrObject* pObj = pMark->GetMarkedSdrObj();
 
-                //	#43984# aktivieren nur, wenn die Maus auch (noch) ueber dem
+                //	aktivieren nur, wenn die Maus auch (noch) ueber dem
                 //	selektierten Objekt steht
 
                 SdrViewEvent aVEvt;
                 SdrHitKind eHit = pView->PickAnything( rMEvt, SDRMOUSEBUTTONDOWN, aVEvt );
                 if ( eHit != SDRHIT_NONE && aVEvt.pObj == pObj )
                 {
-                    UINT16 nSdrObjKind = pObj->GetObjIdentifier();
+                    sal_uInt16 nSdrObjKind = pObj->GetObjIdentifier();
 
                     //
                     //	OLE: aktivieren
@@ -510,7 +530,6 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
                         {
                             if (((SdrOle2Obj*) pObj)->GetObjRef().is())
                             {
-                                //HMHpView->HideMarkHdl();
                                 pViewShell->ActivateObject( (SdrOle2Obj*) pObj, 0 );
                             }
                         }
@@ -518,14 +537,14 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
 
                     //
                     //  Edit text
-                    //  #49458# not in UNO controls
+                    //  not in UNO controls
                     //  #i32352# not in media objects
                     //
                     else if ( pObj->ISA(SdrTextObj) && !pObj->ISA(SdrUnoObj) && !pObj->ISA(SdrMediaObj) )
                     {
                         OutlinerParaObject* pOPO = pObj->GetOutlinerParaObject();
-                        BOOL bVertical = ( pOPO && pOPO->IsVertical() );
-                        USHORT nTextSlotId = bVertical ? SID_DRAW_TEXT_VERTICAL : SID_DRAW_TEXT;
+                        sal_Bool bVertical = ( pOPO && pOPO->IsVertical() );
+                        sal_uInt16 nTextSlotId = bVertical ? SID_DRAW_TEXT_VERTICAL : SID_DRAW_TEXT;
 
                         pViewShell->GetViewData()->GetDispatcher().
                             Execute(nTextSlotId, SFX_CALLMODE_SYNCHRON | SFX_CALLMODE_RECORD);
@@ -538,28 +557,21 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
                             Point aMousePixel = rMEvt.GetPosPixel();
                             pText->SetInEditMode( pObj, &aMousePixel );
                         }
-                        bReturn = TRUE;
+                        bReturn = sal_True;
                     }
                 }
             }
         }
         else if ( TestDetective( pView->GetSdrPageView(), aPnt ) )
-            bReturn = TRUE;
+            bReturn = sal_True;
     }
 
     // Ein VCControl ist aktiv
     // Event an den Manager weiterleiten
     if( bVCAction )
     {
-        //	GetSbxForm gibts nicht mehr - Basic-Controls sind tot
-        //SdrPageView* pPgView = pView->GetPageViewByIndex(0);
-        //ScDrawPage*  pPage	 = (ScDrawPage*)pPgView->GetPage();
-        //VCSbxForm* pForm = (VCSbxForm*)(SbxObject*)(pPage->GetSbxForm());
-        //((VCManager*)(pForm->GetVCContainer()))->
-        //    MouseButtonUp( pWindow, rMEvt );
-        //HMHpView->ShowMarkHdl();
-        bVCAction = FALSE;
-        bReturn = TRUE;
+        bVCAction = false;
+        bReturn = true;
     }
 
     ForcePointer(&rMEvt);
@@ -573,6 +585,18 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
             pViewShell->GetViewData()->GetDispatcher().
                 Execute(SID_OBJECT_SELECT, SFX_CALLMODE_SLOT | SFX_CALLMODE_RECORD);
 
+    if ( bCopy && pViewData && pDocument && pPage )
+    {
+        ScDocShell* pDocShell = pViewData->GetDocShell();
+        ScModelObj* pModelObj = ( pDocShell ? ScModelObj::getImplementation( pDocShell->GetModel() ) : NULL );
+        if ( pModelObj )
+        {
+            SCTAB nTab = pViewData->GetTabNo();
+            ScChartHelper::CreateProtectedChartListenersAndNotify( pDocument, pPage, pModelObj, nTab,
+                aProtectedChartRangesVector, aExcludedChartNames );
+        }
+    }
+
     return (bReturn);
 }
 
@@ -580,14 +604,14 @@ BOOL __EXPORT FuSelection::MouseButtonUp(const MouseEvent& rMEvt)
 |*
 |* Tastaturereignisse bearbeiten
 |*
-|* Wird ein KeyEvent bearbeitet, so ist der Return-Wert TRUE, andernfalls
+|* Wird ein KeyEvent bearbeitet, so ist der Return-Wert sal_True, andernfalls
 |* FALSE.
 |*
 \************************************************************************/
 
-BOOL FuSelection::KeyInput(const KeyEvent& rKEvt)
+sal_Bool FuSelection::KeyInput(const KeyEvent& rKEvt)
 {
-    BOOL bReturn = FALSE;
+    sal_Bool bReturn = false;
 
     if (!bReturn)
     {
@@ -606,27 +630,9 @@ BOOL FuSelection::KeyInput(const KeyEvent& rKEvt)
 
 void FuSelection::Activate()
 {
-/*
-    SdrDragMode eMode;
-    switch (aSfxRequest.GetSlot() )
-    {
-        case SID_OBJECT_SELECT:
-            eMode = SDRDRAG_MOVE;
-            break;
-        case SID_OBJECT_ROTATE:
-            eMode = SDRDRAG_ROTATE;
-            break;
-        case SID_OBJECT_MIRROR:
-            eMode = SDRDRAG_MIRROR;
-            break;
-    }
-    pView->SetDragMode(eMode);
-*/
     FuDraw::Activate();
 }
 
-
-
 /*************************************************************************
 |*
 |* Function deaktivieren
@@ -638,10 +644,6 @@ void FuSelection::Deactivate()
     /**************************************************************************
     * Hide Cursor
     **************************************************************************/
-//    BOOL bShowCursor = FALSE;
-//! pOutlinerView = pView->GetOutlinerView(pWindow, bShowCursor);
-
-//	pView->SetDragMode(SDRDRAG_MOVE);
     FuDraw::Deactivate();
 }
 
@@ -650,8 +652,4 @@ void FuSelection::Deactivate()
 #pragma optimize ( "", on )
 #endif
 
-
-
-
-
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
